﻿using BPA.KitChen.GroupMeal.SqlSugar;
using BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.Manage.Application.AExternalPlatform.Enum;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Material.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Material.Services;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Comm.Enum;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using Org.BouncyCastle.Crypto;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods.Services
{
    public class GoodsService: UserAnalysis, IGoodsService, ITransient
    {
        public GoodsService() 
        {

        }
        /// <summary>
        /// 分页查询商品
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil<List<GoodsDto>>> GetGoodsPageList(GoodsPageInputDto inputDto)
        {
            int total = new RefAsync<int>();
            var data = SqlSugarDb.Db.Queryable<BPA_GoodsInfo, BPA_GoodsType, BPA_GoodsUint>((a, b, c) =>
                new JoinQueryInfos(JoinType.Left, a.GoodsTypeId == b.Id,
                    JoinType.Left, a.GoodsUintId == c.Id))
                .Where((a, b, c) => a.IsDeleted == 0)
                .WhereIF(!string.IsNullOrEmpty(inputDto.GoodsTypeName), (a, b, c) => b.Name.Contains(inputDto.GoodsTypeName))
                .Select((a, b, c) => new GoodsDto()
                {
                    Id =a.Id.SelectAll() ,
                    GoodsTypeName=b.Name,
                    GoodsUnitName=c.Name

                }).ToPageList(inputDto.Current, inputDto.PageSize, ref total);

            return new PageUtil<List<GoodsDto>>()
            {
                Total = total,
                Data = data
            };
        }
        /// <summary>
        /// 添加商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> AddGoods(GoodsInsertDto dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var resEntity = new BPA_GoodsInfo();
                resEntity.GoodsTypeId = dto.GoodsTypeId;
                resEntity.GoodsUintId = dto.GoodsUintId;
                if (string.IsNullOrWhiteSpace(dto.GoodsTypeId))
                {
                    var check=SqlSugarDb.Db.Queryable<BPA_GoodsType>().Where(x => x.Name == "默认分类").First();
                    if (check!=null)
                    {
                        resEntity.GoodsTypeId = check.Id;
                    }
                    else
                    {
                        var GoodsType=await SqlSugarDb.Db.Insertable(new BPA_GoodsType() { Pid="",Name= "默认分类" }).CallEntityMethod(m => m.Create()).ExecuteReturnEntityAsync();
                        resEntity.GoodsTypeId = GoodsType.Id;
                    }
                }
                if (string.IsNullOrWhiteSpace(dto.GoodsUintId))
                {
                    var check = SqlSugarDb.Db.Queryable<BPA_GoodsUint>().Where(x => x.Name == "默认单位").First();
                    if (check != null)
                    {
                        resEntity.GoodsTypeId = check.Id;
                    }
                    else
                    {
                        var GoodsUint = await SqlSugarDb.Db.Insertable(new BPA_GoodsUint() {Name = "默认单位" }).CallEntityMethod(m => m.Create()).ExecuteReturnEntityAsync();
                        resEntity.GoodsUintId = GoodsUint.Id;
                    }
                }
                resEntity.Name = dto.Name;
                resEntity.Descritption = dto.Descritption;
                resEntity.ImgUrl = dto.ImgUrl;
                resEntity.Price = dto.Price;
                resEntity.IsWeigh = dto.IsWeigh;
                
                resEntity.IsAttrubute = true;
                resEntity.Code = GetNumber2(8);
                var res = await SqlSugarDb.Db.Insertable(resEntity).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return res > 0;
            }
            catch (Exception)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(ErrorCodeEnum.Code1007);
            }
           
        }
        /// <summary>
        /// 更新商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateGoods(GoodsUpdateDto dto)
        {
            // 查询数据库中是否存在未删除的商品类型
            var resEntity = SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().Where(it => it.IsDeleted == 0).First(it => it.Id == dto.Id);
            if (null == resEntity)
            {
                throw Oops.Oh(ErrorCodeEnum.Code1008);
            }
            resEntity.GoodsTypeId = dto.GoodsTypeId;
            resEntity.Name = dto.Name;
            resEntity.Descritption = dto.Descritption;
            resEntity.ImgUrl = dto.ImgUrl;
            resEntity.Price = dto.Price;
            resEntity.IsWeigh = dto.IsWeigh;
            resEntity.GoodsUintId = dto.GoodsUintId;
           
            var res = await SqlSugarDb.Db.Updateable(resEntity).ExecuteCommandAsync();
            return res > 0;
        }
        /// <summary>
        /// 删除商品
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        public async Task<bool> DeleteGoods(string[] ids)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var goods = SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().Where(x => ids.Contains(x.Id)).ToList();
                if (goods == null) throw Oops.Oh(ErrorCodeEnum.Code1008);
                var goodsbom = SqlSugarDb.Db.Queryable<BPA_GoodsBom>().Where(x => ids.Contains( x.Goods_Id)).ToList();
                var goodsTechnology = SqlSugarDb.Db.Queryable<BPA_GoodsTechnologyAction>().Where(x => ids.Contains(x.GoodsId)).ToList();
                await SqlSugarDb.Db.Deleteable(goodsbom).ExecuteCommandAsync();
                await SqlSugarDb.Db.Deleteable(goodsTechnology).ExecuteCommandAsync();
                var res = await SqlSugarDb.Db.Deleteable(goods).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return res > 0;
            }
            catch (Exception)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh("删除失败");
            }

        }
        private string GetNumber2(int Length = 10)
        {
            byte[] buffer = Guid.NewGuid().ToByteArray();
            var ram = BitConverter.ToInt64(buffer, 0);
            var str = string.Format("{0}", ram.ToString().Substring(0, Length));
            return str;
        }
    }
}
